package com.capitalware.mqcm;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.*;
import javax.swing.table.*;

import org.jdom.JDOMException;

import com.capitalware.mqcm.models.MQChannelStatusModel;
import com.capitalware.mqcm.mq.PCFChlStatus;
import com.capitalware.mqcm.objects.CWConstants;
import com.capitalware.mqcm.objects.MQObject;
import com.capitalware.mqcm.renderers.RenderRedYellow;

/**
 * MQ Channel Monitor will display a JFrame listing the current channel status.
 * This class is the view ("V") of the MVC.
 *
 * @author Roger Lacroix
 * @version 1.2
 * @license Apache 2 License
 */
public class MQCM extends JFrame implements ComponentListener
{
   private JScrollPane           jSp = null;
   private PCFChlStatus          pcs = null;
   private MQChannelStatusModel  mcsm = null;
   private JTable                table = null;
   //
   private String                qMgrName = "";
   private                       int rowPtr = -1;
   private int                   columnPtr = -1;

   /**
    * Locale specific strings displayed to the user.
    */
   private ResourceBundle labels;

   /**
    * Mainline - entry point
    * @param args
    */
   public static void main(String args[])
   {
      if (args.length == 0)
         new MQCM("mq.xml");
      else
         new MQCM(args[0]);
   }

   /**
    * The constructor
    *
    * @param instance
    */
   public MQCM(String mqXmlFile)
   {
      super();
      
      /**
       * Load up the languages strings we will be using.
       */
      labels = ResourceBundle.getBundle("com.capitalware.mqcm.lang/Lang", Locale.getDefault());
      
      //Turn off metal's use of bold fonts
      UIManager.put("swing.boldMetal", Boolean.FALSE);
      
      addComponentListener(this);

      setSize(CWConstants.MIN_WIDTH, CWConstants.MIN_HEIGHT);     // w * h
      getContentPane().setLayout(new GridBagLayout());

      addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent we)
         {
            dismiss();
         }
      });

      if ( !(System.getProperty("os.name").toLowerCase().startsWith("linux")) )
      {
         try
         {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
         } catch (Exception ex) {}
      }

      setTitle(labels.getString("mqcm.name") + " v" + CWConstants.VERSION);
      ImageIcon icon = new ImageIcon(MQCM.class.getResource("images/mqcm32.gif"));
      this.setIconImage(icon.getImage());

      this.setVisible(true);
      validate();

      MQObject mqo = new MQObject(mqXmlFile);
      try
      {
         mqo.checkQM();
         qMgrName = mqo.qMgrName;
         pcs = new PCFChlStatus(mqo, MQCM.this, labels);
         pcs.start();
      }
      catch (IOException e)
      {
         JOptionPane.showMessageDialog( this,
                                        e.getLocalizedMessage(),
                                        labels.getString("popup.error"),
                                        JOptionPane.ERROR_MESSAGE);
         dismiss();
      }
      catch (JDOMException e)
      {
         JOptionPane.showMessageDialog( this,
                                        e.getLocalizedMessage(),
                                        labels.getString("popup.error"),
                                        JOptionPane.ERROR_MESSAGE);
         dismiss();
      }
      catch (IllegalArgumentException e)
      {
         JOptionPane.showMessageDialog( this,
                                        e.getLocalizedMessage(),
                                        labels.getString("popup.error"),
                                        JOptionPane.ERROR_MESSAGE);
         dismiss();
      }
   }

   /**
    * Size out the columns of the JTable.
   */
   private void sizeOutColumns()
   {
      TableColumnModel columnModel = table.getColumnModel();

      /*
       * Create a cell renderer that aligns the text to the left.
       */
      DefaultTableCellRenderer dtcr1 = new DefaultTableCellRenderer()
      {
         public void setValue(Object value)
         {
            super.setValue(value);
         }
      };
      dtcr1.setHorizontalAlignment(DefaultTableCellRenderer.LEFT);

      /*
       * Create a cell renderer that centers the text.
       */
      DefaultTableCellRenderer dtcr2 = new DefaultTableCellRenderer()
      {
         public void setValue(Object value)
         {
            super.setValue(value);
         }
      };
      dtcr2.setHorizontalAlignment(DefaultTableCellRenderer.CENTER);

      /*
       * Create a cell renderer that will colorize the statuses and
       * centers the text
       */
      RenderRedYellow rrg = new RenderRedYellow(labels);

      /*
       * Rssize the columns
       */
      for (int i=0; i < CWConstants.columnWidth.length; i++)
      {
         TableColumn column = columnModel.getColumn(i);
         column.setPreferredWidth(CWConstants.columnWidth[i]);
         column.setMaxWidth(CWConstants.columnWidth[i]);

         if (i == 0)
            column.setCellRenderer(dtcr1);
         else if ( (i == 2) || (i == 13) )
            column.setCellRenderer(rrg);
         else
            column.setCellRenderer(dtcr2);
      }

      columnModel = null;
   }

   /**
    * Update the table data with new data.
    * @param al
    */
   public void updateTable(ArrayList<Object> al)
   {
      GridBagConstraints gbc = new GridBagConstraints();
      SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");

      if (table != null)
      {
         rowPtr = table.getSelectedRow();
         columnPtr = table.getSelectedColumn();
      }

      if (mcsm == null)
      {
         /*
          * First time through the updateTable() method
          */
         mcsm = new MQChannelStatusModel(al, labels);
         table = new JTable(mcsm);
         table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
         // Disable auto resizing to make the table horizontal scrollable
         table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
         // Do not allow the User to Move a Column in a JTable Component
         table.getTableHeader().setReorderingAllowed(false);

         jSp   = new JScrollPane(table);

         gbc.anchor = GridBagConstraints.CENTER;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.weightx = 1;
         gbc.weighty = 1;
         gbc.insets = new Insets(0,0,0,0);
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.gridwidth = 1;
         gbc.gridheight = 100;
         getContentPane().add(jSp, gbc);

         ((DefaultTableCellRenderer)table.getTableHeader().getDefaultRenderer()).setHorizontalAlignment(SwingConstants.CENTER);

         sizeOutColumns( );

         // force the pane to be updated.
         this.setVisible(true);
         repaint();
      }
      else
      {
         mcsm.setTableData(al);

         SwingUtilities.invokeLater(new Runnable()
         {
            public void run()
            {
               mcsm.fireTableDataChanged();

               if ( (rowPtr != -1) && (table.getRowCount() >= rowPtr) )
                  table.changeSelection(rowPtr, columnPtr, false, false);
            }
         });
      }

      setTitle(labels.getString("mqcm.name") + " : " + qMgrName + " : " + labels.getString("popup.title.lastupdateat") + " " + timeFormat.format(new java.util.Date()));
   }

   /**
    * The user is resizing the component
    * @param e
    */
   public void componentResized(ComponentEvent e)
   {
      boolean resize = false;
      int width = getWidth();
      int height = getHeight();

      // Check if either the width or the height are below minimum

      if (width < CWConstants.MIN_WIDTH)
      {
         resize = true;
         width = CWConstants.MIN_WIDTH;
      }

      if (height < CWConstants.MIN_HEIGHT)
      {
         resize = true;
         height = CWConstants.MIN_HEIGHT;
      }

      if (resize)
         setSize(width, height);
   }

   public void componentMoved(ComponentEvent e)
   {}

   public void componentShown(ComponentEvent e)
   {}

   public void componentHidden(ComponentEvent e)
   {}

   /**
    * Time to cleanup the JFrame
    */
   void dismiss()
   {
      if (pcs != null)
      {
         pcs.stopPlease();
         pcs.interrupt();
      }

      this.setVisible(false);

      // wait for the MQ child thread to end.
      try { Thread.sleep(300); } catch (InterruptedException ie) {}

      this.dispose();
   }
}
