package com.capitalware.mqcm.xml;

import java.io.*;

import org.jdom.*;
import org.jdom.input.*;

import com.capitalware.mqcm.objects.CWConstants;
import com.capitalware.mqcm.xml.DTD.ResolveLocalDTD;

/**
 * This class handles the reading and parsing of MQ XML file.
 *
 * @author Roger Lacroix
 * @version 1.2
 * @license Apache 2 License
 */
public class MQXml
{
   public static final String  MQ_PATH = "mq" + File.separator;

   private final String MONITOR_CHL_NAME_ELEMENT   = "MonitorChannelName";
   private final String REFRESH_RATE_ELEMENT       = "RefreshRate";
   private final String CCDTFILE_ELEMENT           = "CCDTFile";
   private final String QMGRNAME_ELEMENT           = "QMgrName";
   private final String HOSTNAME_ELEMENT           = "Hostname";
   private final String PORT_ELEMENT               = "Port";
   private final String CHANNELNAME_ELEMENT        = "ChannelName";
   private final String USERID_ELEMENT             = "UserID";
   private final String PASSWORD_ELEMENT           = "Password";
   private final String SECURITYEXIT_ELEMENT       = "SecurityExit";
   private final String SECURITYEXITPATH_ELEMENT   = "SecurityExitPath";
   private final String CIPHERSUITENAME_ELEMENT    = "CipherSuiteName";
   private final String DISTINGUISHEDNAME_ELEMENT  = "DistinguishedName";
   private final String TRUSTEDSTORE_ELEMENT       = "TrustedStore";
   private final String TRUSTEDSTOREPASSWD_ELEMENT = "TrustedStorePasswd";
   private final String KEYSTORE_ELEMENT           = "KeyStore";
   private final String KEYSTOREPASSWD_ELEMENT     = "KeyStorePasswd";
   private final String LDAPSERVER_ELEMENT         = "LDAPServer";
   private final String LDAPSERVERPORT_ELEMENT     = "LDAPServerPort";

   private String _qMgrName = null;
   private String _monitorChlName = null;
   private String _refreshRate = null;
   private String _fileCCDT = null;
   private String _hostname = null;
   private String _channelName = null;
   private String _port = null;
   private String _userID = null;
   private String _password = null;
   private String _securityExit = null;
   private String _securityExitPath = null;
   //
   private String  _cipherSuiteName = null;
   private String  _distinguishedName = null;
   private String  _trustedStore = null;
   private String  _trustedStorePasswd = null;
   private String  _keyStore = null;
   private String  _keyStorePasswd = null;
   private String  _ldapServer = null;
   private String  _ldapServerPort = null;

   /**
    * The constructor
    */
   public MQXml()
   {
      super();
   }

   /**
    * Read MQ XML file and set parameters
    *
    * @param xmlFilename
    *           Filename of mq.xml file
    * @throws IOException
    */
   public void readXMLFile(String xmlFileName) throws JDOMException, IOException
   {
      SAXBuilder builder = new SAXBuilder(true);
      // handle the ability to pull DTD from Jar if needed
      builder.setEntityResolver(new ResolveLocalDTD(CWConstants.MQ_DTD));
      Document doc = builder.build(MQ_PATH + xmlFileName);
      Element rootElement = doc.getRootElement();

      if ( rootElement.getChild(MONITOR_CHL_NAME_ELEMENT) != null)
         _monitorChlName = rootElement.getChild(MONITOR_CHL_NAME_ELEMENT).getTextTrim();

      if ( rootElement.getChild(REFRESH_RATE_ELEMENT) != null)
         _refreshRate = rootElement.getChild(REFRESH_RATE_ELEMENT).getTextTrim();

      if ( rootElement.getChild(QMGRNAME_ELEMENT) != null)
         _qMgrName = rootElement.getChild(QMGRNAME_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CCDTFILE_ELEMENT) != null)
         _fileCCDT = rootElement.getChild(CCDTFILE_ELEMENT).getTextTrim();

      if ( rootElement.getChild(HOSTNAME_ELEMENT) != null)
         _hostname = rootElement.getChild(HOSTNAME_ELEMENT).getTextTrim();

      if ( rootElement.getChild(PORT_ELEMENT) != null)
         _port = rootElement.getChild(PORT_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CHANNELNAME_ELEMENT) != null)
         _channelName = rootElement.getChild(CHANNELNAME_ELEMENT).getTextTrim();

      if ( rootElement.getChild(USERID_ELEMENT) != null)
         _userID = rootElement.getChild(USERID_ELEMENT).getTextTrim();

      if ( rootElement.getChild(PASSWORD_ELEMENT) != null)
         _password = rootElement.getChild(PASSWORD_ELEMENT).getTextTrim();

      if ( rootElement.getChild(SECURITYEXIT_ELEMENT) != null)
         _securityExit = rootElement.getChild(SECURITYEXIT_ELEMENT).getTextTrim();

      if ( rootElement.getChild(SECURITYEXITPATH_ELEMENT) != null)
         _securityExitPath = rootElement.getChild(SECURITYEXITPATH_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CIPHERSUITENAME_ELEMENT) != null)
         _cipherSuiteName = rootElement.getChild(CIPHERSUITENAME_ELEMENT).getTextTrim();

      if ( rootElement.getChild(DISTINGUISHEDNAME_ELEMENT) != null)
         _distinguishedName = rootElement.getChild(DISTINGUISHEDNAME_ELEMENT).getTextTrim();

      if ( rootElement.getChild(TRUSTEDSTORE_ELEMENT) != null)
         _trustedStore = rootElement.getChild(TRUSTEDSTORE_ELEMENT).getTextTrim();

      if ( rootElement.getChild(TRUSTEDSTOREPASSWD_ELEMENT) != null)
         _trustedStorePasswd = rootElement.getChild(TRUSTEDSTOREPASSWD_ELEMENT).getTextTrim();

      if ( rootElement.getChild(KEYSTORE_ELEMENT) != null)
         _keyStore = rootElement.getChild(KEYSTORE_ELEMENT).getTextTrim();

      if ( rootElement.getChild(KEYSTOREPASSWD_ELEMENT) != null)
         _keyStorePasswd = rootElement.getChild(KEYSTOREPASSWD_ELEMENT).getTextTrim();

      if ( rootElement.getChild(LDAPSERVER_ELEMENT) != null)
         _ldapServer = rootElement.getChild(LDAPSERVER_ELEMENT).getTextTrim();

      if ( rootElement.getChild(LDAPSERVERPORT_ELEMENT) != null)
         _ldapServerPort = rootElement.getChild(LDAPSERVERPORT_ELEMENT).getTextTrim();
   }

   /**
    * Retrieve the CCDT file.
    * @return fileCCDT
    */
   public String getCCDTFile()
   {
      return _fileCCDT;
   }

   /**
    * Retrieve the queue manager name.
    * @return qMgrName
    */
   public String getQMgrName()
   {
      return _qMgrName;
   }

   /**
    * Retrieve the _monitor channel name.
    * @return monitorChlName
    */
   public String getMonitorChannelName()
   {
      if (_monitorChlName != null)
         return _monitorChlName;
      else
         return "*";
   }

   /**
    * Retrieve the refresh rate in seconds.
    * @return refreshRate
    */
   public int getRefreshRate()
   {
      int refreshRate = 60;
      try
      {
         refreshRate = Integer.parseInt(_refreshRate);
      }
      catch (NumberFormatException nfe)
      {
         refreshRate = 60;
      }

      return refreshRate;
   }

   /**
    * Retrieve the hostname.
    * @return hostname
    */
   public String getHostname()
   {
      return _hostname;
   }

   /**
    * Retrieve the channel name.
    * @return channelName
    */
   public String getChannelName()
   {
      return _channelName;
   }

   /**
    * Retrieve the port number.
    * @return portnumber
    */
   public int getPort()
   {
      int portNum = 1414;
      try
      {
         portNum = Integer.parseInt(_port);
      }
      catch (NumberFormatException nfe)
      {
         portNum = -1;
      }

      return portNum;
   }

   /**
    * Retrieve the UserID.
    * @return userID
    */
   public String getUserID()
   {
      return _userID;
   }

   /**
    * Retrieve the Password.
    * @return password
    */
   public String getPassword()
   {
      return _password;
   }

   /**
    * Retrieve the security exit name.
    * @return securityExit
    */
   public String getSecurityExit()
   {
      return _securityExit;
   }

   /**
    * Retrieve the security exit path.
    * @return securityExitPath
    */
   public String getSecurityExitPath()
   {
      return _securityExitPath;
   }

   /**
    * Retrieve the CipherSuiteName.
    * @return cipherSuiteName
    */
   public String getCipherSuiteName()
   {
      return _cipherSuiteName;
   }

   /**
    * Retrieve the Distinguished Name.
    * @return distinguishedName
    */
   public String getDistinguishedName()
   {
      return _distinguishedName;
   }

   /**
    * Retrieve the Trusted Store.
    * @return trustedStore
    */
   public String getTrustedStore()
   {
      return _trustedStore;
   }

   /**
    * Retrieve the Trusted Store Password.
    * @return trustedStorePasswd
    */
   public String getTrustedStorePasswd()
   {
      return _trustedStorePasswd;
   }

   /**
    * Retrieve the KeyStore.
    * @return keyStore
    */
   public String getKeyStore()
   {
      return _keyStore;
   }

   /**
    * Retrieve the Key Store Password.
    * @return keyStorePasswd
    */
   public String getKeyStorePasswd()
   {
      return _keyStorePasswd;
   }

   /**
    * Retrieve the LDAP Server.
    * @return ldapServer
    */
   public String getLdapServer()
   {
      return _ldapServer;
   }

   /**
    * Retrieve the LDAP Server Port number.
    * @return portNum
    */
   public int getLdapServerPort()
   {
      int portNum = 389;
      try
      {
         portNum = Integer.parseInt(_ldapServerPort);
      }
      catch (NumberFormatException nfe)
      {
         portNum = -1;
      }

      return portNum;
   }
}
